from blank_portfolio.models.data_models import Skill, SkillLevel
from typing import List, Literal


def sort_skills_by_level(skills: List[Skill]) -> List[Skill]:

    skills_sorted = sorted(skills, key=lambda s: s.level.value)
    return skills_sorted


def filter_skills(
    skills: List[Skill],
    names: List[str] = None,
    types: List[str] = None,
    levels: List[str] = None,
) -> List[Skill]:
    filtered_skills = skills
    if names:
        names = [n.lower() for n in names]
        filtered_skills = [
            skill for skill in filtered_skills if skill.name.lower() in names
        ]

    if types:
        types = [t.upper() for t in types]
        filtered_skills = [
            skill for skill in filtered_skills if skill.type.name in types
        ]

    if levels:
        levels = [l.upper() for l in levels]
        filtered_skills = [
            skill for skill in filtered_skills if skill.level.name in levels
        ]

    return filtered_skills



import click
from blank_portfolio.utils.formatter_utils import (
    print_splash,
    print_portfolio,
    print_skills,
    print_experiences,
)
from blank_portfolio.services.cli_service import CLIService

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    """🛠️ Blank Portfolio CLI: Explore and filter your developer portfolio from the terminal."""
    print_splash()


@cli.command()
def show_portfolio():
    """Show the portfolio summary including bio, skills, and projects."""
    print_portfolio()


@cli.command(
    help="Show the list of skills with optional filters on name, type, level, and sort order."
)
@click.option(
    "--skill-name",
    "-sn",
    help="Filter skills by comma-separated name(s), e.g., 'skill1,skill2'.",
)
@click.option(
    "--skill-type",
    "-st",
    multiple=True,
    help="Filter skills by type(s). Multiple types can be specified using repeated flags, e.g., -st Tool -st Framework.",
)
@click.option(
    "--skill-level",
    "-sl",
    multiple=True,
    help="Filter skills by level(s). Multiple levels are specified using repeated flags, e.g., -sl Expert -sl Beginner.",
)
@click.option(
    "--sort",
    "-s",
    type=click.Choice(["asc", "desc"], case_sensitive=False),
    default=None,
    help="Sort skills by level ascending ('asc') or descending ('desc').",
)
def list_skills(skill_name, skill_type, skill_level, sort):
    """List your skills filtered by provided criteria."""
    # Process comma-separated skill names into list if provided
    if skill_name:
        skill_name = list(set(skill_name.split(",")))
    if skill_level:
        skill_level = list(set(skill_level))
    if skill_type:
        skill_type = list(set(skill_type))
    print_skills(
        skill_name=skill_name, skill_type=skill_type, skill_level=skill_level, sort=sort
    )


@cli.command(help="Show the list of experiences in your portfolio.")
def list_exp():
    print_experiences()



from datetime import date
from typing import List
from blank_portfolio.models.data_models import *
from blank_portfolio.utils.data_utils import sort_skills_by_level


class DataSource_provider:
    _instance = None

    def __init__(self):
        if self._instance is not None:
            return
        else:
            self._instance = self.generate_portfolio()

    def get_instance(self):
        return self._instance if self._instance else self.__init__()

    def generate_portfolio(self) -> Portfolio:
        _name = "Shantanu Bindhani"
        _role = "SDE / Backend Developer / Software Engineer"
        _bio = [
            "Hi, I am Software Developer from hailing from Balasore, Orissa, India.",
            "I am Experienced in backend development, microservices, \n  and also knowledgeable about Web, Game, Android, software development.",
            "Some of my strongest holds are on python, java, c++, and other technologies related to them.\n you can skim through my portfolio to learn more about them.",
        ]
        _links = self.get_links()
        _contacts = self.get_contacts()
        _experiences = self.get_experiences()
        _projects = self.get_projects()
        _educations = self.get_educations()
        _skills = self.get_skills()
        _certifications = self.get_certifications()

        _portfolio = Portfolio(
            name=_name,
            role=_role,
            bio=_bio,
            links=_links,
            contacts=_contacts,
            experiences=_experiences,
            projects=_projects,
            educations=_educations,
            skills=_skills,
            certifications=_certifications,
        )

        return _portfolio

    def get_links(self) -> List[Link]:
        return [
            Link(url="https://linkedin.com/in/shantanubindhani", label="LinkedIn"),
            Link(url="https://github.com/shantanubindhani", label="GitHub"),
        ]

    def get_contacts(self) -> List[Contact]:
        return [
            Contact(detail="shantanubindhani1805@gmail.com", type=ContactType.EMAIL),
            Contact(detail="+91-8917322838", type=ContactType.PHONE),
        ]

    def get_experiences(self) -> List[Experience]:
        return [
            Experience(
                role="Backend SDE Intern",
                company="Datachecks, Bangalore",
                mode=JobMode.REMOTE,
                start_date=date(2024, 11, 1),
                end_date=date(2025, 4, 30),
                responsibilities=[
                    "Spearheaded the development and enhancement of multiple features and system integrations using a modern Python tech stack, specifically leveraging FastAPI for high-performance API creation, SQL Alchemy for robust ORM, and Celery for efficient asynchronous task processing, all connected to a PostgreSQL database.",
                    "Ensured code reliability and functional correctness by comprehensively designing and writing automated tests at multiple levels, including Unit tests, Integration, and Repository tests, utilizing the Pytest framework to maintain a high standard of quality assurance.",
                    "Applied advanced software engineering principles and system design patterns, such as the Singleton Architecture and Dependency Injection, to optimize code structure and application flow, directly resulting in significant improvements to overall efficiency and reduced load times.",
                    "Actively monitored and maintained the stability and reliability of the deployed APIs and application services; specifically utilized Sentry for real-time error tracking and alerting, and leveraged Codecov to track and enforce high API test coverage metrics.",
                    "Engaged in end-to-end feature development within a small, highly collaborative and qualified team, taking ownership from conceptual design through deployment, and facilitating efficient knowledge sharing and code review processes to ensure alignment with project goals and delivery timelines.",
                ],
                learnings=[
                    "Learned to build and deploy high-performance Python APIs, mastering the FastAPI, SQL Alchemy, and Celery stack with PostgreSQL.",
                    "Gained expertise in test-driven development, mastering the creation of Unit, Integration, and Repository tests using the Pytest framework.",
                    "Acquired practical skills in application monitoring and quality assurance using Sentry for real-time tracking and Codecov for test coverage enforcement.",
                    "Mastered the application of advanced design patterns (Singleton Architecture, Dependency Injection) to optimize code for efficiency and reduced load times.",
                ],
            ),
            Experience(
                role="Software Development Engineer Intern",
                company="Ciright Inc., Ahmedabad",
                mode=JobMode.ONSITE,
                start_date=date(2023, 1, 1),
                end_date=date(2023, 6, 30),
                responsibilities=[
                    "Developed and maintained 30+ business verticals for platforms like Ciright.com and Veuit.com, utilizing a Java Microservices architecture for scalable back-end development.",
                    "Drove significant system efficiency and modernization by implementing Spring Boot 3 and leveraging multithreading techniques, resulting in a 20% overall performance improvement.",
                    "Handled the entire data and presentation layer, implementing the Hibernate ORM with H2DB for data persistence and integrating back-end services with the UI built using JSP (Java Full Stack).",
                    "Designed and enforced core application structure by utilizing the Spring framework and applying industry-standard patterns like the Java Singleton Architecture for resource management.",
                    "Contributed to specialized technical requirements, including complex report generation using JRXML and the implementation of security components using C++ (STL) and cryptography principles.",
                ],
                learnings=[
                    "Learned to develop and scale enterprise applications by applying Java Microservices architecture with Spring and Spring Boot.",
                    "Gained critical knowledge in performance tuning, specifically how to achieve 20% system efficiency gains using multithreading and modern Spring Boot 3 features.",
                    "Acquired full-stack proficiency, learning to manage data persistence with Hibernate and H2DB, and integrating the back-end with the JSP view layer.",
                    "Developed foundational versatility by learning specialized tools like JRXML for reporting and implementing low-level concepts using C++ (STL) and cryptography.",
                ],
            ),
        ]

    def get_projects(self) -> List[Project]:
        return [
            Project(
                name="Expense Tracker",
                description="Android app to track daily and monthly expenses using Kotlin, Jetpack Compose, MVVM, Room-DB, Coroutines.",
                technologies=[
                    "Kotlin",
                    "Jetpack Compose",
                    "MVVM",
                    "Room-DB",
                    "Coroutines",
                ],
                link="https://github.com/shantanubindhani/Expense-tracker",
            ),
            Project(
                name="Melodic Hues",
                description="Web app recommending songs based on facial expressions using Deep Learning and Flask, supporting multiple languages.",
                technologies=["Python", "Flask", "CNN", "Deep Learning"],
                link="https://github.com/shantanubindhani/Melodic-Hues",
            ),
        ]

    def get_educations(self) -> List[Education]:
        return [
            Education(
                degree="Masters in Computer Applications",
                grade=Grade(score=8.68, scale=10.0),
                institution="Lovely Professional University, Punjab",
                graduation_date=date(2025, 5, 1),
                additional_info=None,
            ),
            Education(
                degree="Bachelors in Computer Application",
                grade=Grade(score=8.75, scale=10.0),
                institution="Rai University, Ahmedabad",
                graduation_date=date(2023, 5, 1),
                additional_info=None,
            ),
        ]

    def get_skills(self) -> List[Skill]:
        _skills = [
            Skill(name="Python", type=SkillType.LANGUAGE, level=SkillLevel.EXPERT),
            Skill(name="C++", type=SkillType.LANGUAGE, level=SkillLevel.ADVANCED),
            Skill(name="Java", type=SkillType.LANGUAGE, level=SkillLevel.ADVANCED),
            Skill(name="Kotlin", type=SkillType.LANGUAGE, level=SkillLevel.BEGINNER),
            Skill(
                name="JavaScript",
                type=SkillType.LANGUAGE,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(name="SQL", type=SkillType.TECHNOLOGY, level=SkillLevel.ADVANCED),
            Skill(name="Git", type=SkillType.TOOL, level=SkillLevel.EXPERT),
            Skill(name="BASH", type=SkillType.TOOL, level=SkillLevel.ADVANCED),
            Skill(name="FastAPI", type=SkillType.FRAMEWORK, level=SkillLevel.ADVANCED),
            Skill(
                name="Spring Boot",
                type=SkillType.FRAMEWORK,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(name="Docker", type=SkillType.TOOL, level=SkillLevel.INTERMEDIATE),
            Skill(name="Kubernetes", type=SkillType.TOOL, level=SkillLevel.BEGINNER),
            Skill(name="Celery", type=SkillType.TOOL, level=SkillLevel.INTERMEDIATE),
            Skill(
                name="PostgreSQL", type=SkillType.TECHNOLOGY, level=SkillLevel.ADVANCED
            ),
            Skill(
                name="REST APIs", type=SkillType.TECHNOLOGY, level=SkillLevel.ADVANCED
            ),
            Skill(
                name="Microservices",
                type=SkillType.TECHNOLOGY,
                level=SkillLevel.ADVANCED,
            ),
            Skill(name="JUnit", type=SkillType.TOOL, level=SkillLevel.BEGINNER),
            Skill(
                name="JWTAuth", type=SkillType.TECHNOLOGY, level=SkillLevel.INTERMEDIATE
            ),
            Skill(
                name="Android Development",
                type=SkillType.TECHNOLOGY,
                level=SkillLevel.BEGINNER,
            ),
            Skill(
                name="Jetpack Compose",
                type=SkillType.FRAMEWORK,
                level=SkillLevel.BEGINNER,
            ),
            Skill(
                name="Coroutines", type=SkillType.TECHNOLOGY, level=SkillLevel.BEGINNER
            ),
            Skill(
                name="Flask", type=SkillType.FRAMEWORK, level=SkillLevel.INTERMEDIATE
            ),
            Skill(name="CNN", type=SkillType.TECHNOLOGY, level=SkillLevel.BEGINNER),
            Skill(
                name="Deep Learning",
                type=SkillType.TECHNOLOGY,
                level=SkillLevel.BEGINNER,
            ),
            Skill(name="Pytest", type=SkillType.TOOL, level=SkillLevel.ADVANCED),
            Skill(
                name="System Design",
                type=SkillType.CONCEPT,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(
                name="Multiprocessing",
                type=SkillType.TECHNOLOGY,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(name="Sentry", type=SkillType.TOOL, level=SkillLevel.BEGINNER),
            Skill(name="Codecov", type=SkillType.TOOL, level=SkillLevel.BEGINNER),
            Skill(
                name="Python-Typing", type=SkillType.CONCEPT, level=SkillLevel.BEGINNER
            ),
            Skill(
                name="Data Structures & Algorithms",
                type=SkillType.CONCEPT,
                level=SkillLevel.ADVANCED,
            ),
            Skill(
                name="OOPs Concepts", type=SkillType.CONCEPT, level=SkillLevel.EXPERT
            ),
            Skill(
                name="Design Patterns",
                type=SkillType.CONCEPT,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(
                name="Agile", type=SkillType.METHODOLOGY, level=SkillLevel.INTERMEDIATE
            ),
            Skill(
                name="Scrum", type=SkillType.METHODOLOGY, level=SkillLevel.INTERMEDIATE
            ),
            Skill(
                name="Unit Testing",
                type=SkillType.METHODOLOGY,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(
                name="Integration Testing",
                type=SkillType.METHODOLOGY,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(
                name="Repository Testing",
                type=SkillType.METHODOLOGY,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(
                name="Multithreading", type=SkillType.CONCEPT, level=SkillLevel.BEGINNER
            ),
            Skill(
                name="JRXML", type=SkillType.TECHNOLOGY, level=SkillLevel.INTERMEDIATE
            ),
            Skill(
                name="CI/CD", type=SkillType.METHODOLOGY, level=SkillLevel.INTERMEDIATE
            ),
            Skill(
                name="Github Workflows",
                type=SkillType.TOOL,
                level=SkillLevel.INTERMEDIATE,
            ),
            Skill(name="Linux", type=SkillType.TECHNOLOGY, level=SkillLevel.ADVANCED),
            Skill(name="VS Code", type=SkillType.TOOL, level=SkillLevel.EXPERT),
            Skill(name="Eclipse", type=SkillType.TOOL, level=SkillLevel.ADVANCED),
            Skill(
                name="Android Studio", type=SkillType.TOOL, level=SkillLevel.BEGINNER
            ),
            Skill(name="Postman", type=SkillType.TOOL, level=SkillLevel.INTERMEDIATE),
            Skill(name="DBeaver", type=SkillType.TOOL, level=SkillLevel.INTERMEDIATE),
            Skill(name="C#", type=SkillType.LANGUAGE, level=SkillLevel.BEGINNER),
            Skill(
                name="HTML/CSS", type=SkillType.TECHNOLOGY, level=SkillLevel.ADVANCED
            ),
            Skill(name="ReactJS", type=SkillType.FRAMEWORK, level=SkillLevel.BEGINNER),
            Skill(name="GitHub", type=SkillType.TOOL, level=SkillLevel.ADVANCED),
        ]

        return _skills

    def get_certifications(self) -> List[Certification]:
        return [
            Certification(
                name="Java (Basic) Certificate",
                organization="Hackerrank",
                issue_date=date(2024, 8, 1),
                expiration_date=None,
                url="https://www.hackerrank.com/certificates/3fbe9c798cd2",
            ),
            Certification(
                name="Software Engineer Certificate",
                organization="Hackerrank",
                issue_date=date(2024, 8, 1),
                expiration_date=None,
                url="https://www.hackerrank.com/certificates/3760edda3156",
            ),
            Certification(
                name="Mastering Android Development with Kotlin From Beginner to Pro Certification",
                organization="GFG",
                issue_date=date(2024, 6, 1),
                expiration_date=date(2024, 7, 31),
                url="https://www.geeksforgeeks.org/certificate/bdb05332ac99bedba18c420cbb25445f",
            ),
            Certification(
                name="Jetpack Compose & Kotlin & Java for Android App Development Certification",
                organization="Udemy",
                issue_date=date(2024, 4, 1),
                expiration_date=date(2024, 7, 31),
                url="https://www.udemy.com/certificate/UC-f88e1672-eaa9-4442-a3aa-e07c0f69468c/",
            ),
            Certification(
                name="The Complete Python Developer Certification",
                organization="Udemy",
                issue_date=date(2024, 1, 1),
                expiration_date=date(2024, 6, 30),
                url="https://www.udemy.com/certificate/UC-0096b90f-42cf-4135-9b89-947b437d09bd/",
            ),
        ]


portfolio = DataSource_provider().get_instance()



from typing import List
from blank_portfolio.models.data_models import *
from blank_portfolio.utils.data_utils import sort_skills_by_level, filter_skills
from blank_portfolio.data.datasource import DataSource_provider


class CLIService:
    def __init__(self, portfolio: Portfolio = None):
        self._portfolio = portfolio
        if not self._portfolio:
            self._portfolio = DataSource_provider().get_instance()

    def get_portfolio(self) -> Portfolio:
        return self._portfolio

    def get_contacts(self) -> List[Contact]:
        return self._portfolio.contacts

    def get_experiences(self) -> List[Experience]:
        return self._portfolio.experiences

    def get_skills(self) -> List[Skill]:
        return self._portfolio.skills

    def get_skills_sorted_by_level(self) -> List[Skill]:
        return sort_skills_by_level(self._portfolio.skills)

    def get_skills_filtered(
        self,
        skill_types: List[str],
        skill_names: List[str],
        skill_levels: List[str],
        sort: str = None,
    ) -> List[Skill]:
        _skills = self._portfolio.skills
        if sort == "asc":
            _skills = self.get_skills_sorted_by_level()[::-1]
        elif sort == "desc":
            _skills = self.get_skills_sorted_by_level()

        return filter_skills(
            skills=_skills,
            names=skill_names,
            types=skill_types,
            levels=skill_levels,
        )



from enum import Enum
from datetime import date
from dataclasses import dataclass
from typing import List, Optional, Union


class JobMode(Enum):
    REMOTE = "REMOTE"
    ONSITE = "ONSITE"
    HYBRID = "HYBRID"


class ContactType(Enum):
    EMAIL = "EMAIL"
    PHONE = "PHONE"
    OTHER = "OTHER"
    GITHUB = "GITHUB"
    LINKEDIN = "LINKEDIN"


class SkillLevel(Enum):
    BEGINNER = 1
    INTERMEDIATE = 2
    ADVANCED = 3
    EXPERT = 4


class SkillType(Enum):
    LANGUAGE = "LANGUAGE"
    FRAMEWORK = "FRAMEWORK"
    TOOL = "TOOL"
    METHODOLOGY = "METHODOLOGY"
    CONCEPT = "CONCEPT"
    TECHNOLOGY = "TECHNOLOGY"
    OTHER = "OTHER"


@dataclass
class Link:
    url: str
    label: str

    def __post_init__(self):
        if not self.url or not self.label:
            raise ValueError(f"Link url and label attributes cannot be left empty!.")


@dataclass
class Contact:
    detail: str
    type: ContactType

    def __post_init__(self):
        if not self.type or not self.detail:
            raise ValueError("Contact Type and Detail cannot be left empty!.")


@dataclass
class Grade:
    score: float
    scale: float
    details: Optional[str] = None

    def __post_init__(self):
        if not self.score or not self.scale:
            raise ValueError("Grade Score and Scale cannot be left empty!.")


@dataclass
class Education:
    degree: str
    grade: Grade
    institution: str
    graduation_date: date
    additional_info: Optional[str] = None

    def __post_init__(self):
        err_str = (
            "" + "degree, "
            if not self.degree
            else (
                "" + "grade, "
                if not self.grade
                else (
                    "" + "institution, "
                    if not self.institution
                    else "" + "graduation_date, " if not self.graduation_date else ""
                )
            )
        )
        if err_str != "":
            raise ValueError("Education (" + err_str[:-2] + ") cannot be left empty!.")


@dataclass
class Project:
    name: str
    description: str
    technologies: List[str]
    link: Optional[str] = None

    def __post_init__(self):
        err_str = (
            "" + "name, "
            if not self.name
            else (
                "" + "description, "
                if not self.description
                else "" + "technologies, " if not self.technologies else ""
            )
        )
        if err_str != "":
            raise ValueError("Project (" + err_str[:-2] + ") cannot be left empty!.")


@dataclass
class Skill:
    name: str
    type: SkillType
    level: Optional[SkillLevel] = None

    def __post_init__(self):
        if not self.type:
            self.type = SkillType.OTHER
        if not self.level:
            self.level = SkillLevel.BEGINNER


@dataclass
class Experience:
    role: str
    company: str
    mode: JobMode
    start_date: date
    responsibilities: List[str]
    end_date: Optional[date] = None
    learnings: Optional[List[str]] = None

    def __post_init__(self):
        err_str = (
            "" + "role, "
            if not self.role
            else (
                "" + "company, "
                if not self.company
                else (
                    "" + "mode, "
                    if not self.mode
                    else (
                        "" + "start_date, "
                        if not self.start_date
                        else (
                            "" + "responsibilities, "
                            if not self.responsibilities
                            else ""
                        )
                    )
                )
            )
        )
        if err_str != "":
            raise ValueError("Experience (" + err_str[:-2] + ") cannot be left empty!.")


@dataclass
class Certification:
    url: str
    name: str
    organization: str
    issue_date: date
    expiration_date: Optional[date] = None

    def __post_init__(self):
        err_str = (
            "" + "url, "
            if not self.url
            else (
                "" + "name, "
                if not self.name
                else (
                    "" + "organization, "
                    if not self.organization
                    else "" + "issue_date, " if not self.issue_date else ""
                )
            )
        )
        if err_str != "":
            raise ValueError(
                "Certification (" + err_str[:-2] + ") cannot be left empty!."
            )


@dataclass
class Portfolio:
    name: str
    role: str
    bio: Union[List[str], str]
    links: List[Link]
    contacts: List[Contact]
    experiences: List[Experience]
    projects: List[Project]
    educations: List[Education]
    skills: List[Skill]
    certifications: List[Certification]

    def __post_init__(self):
        err_str = (
            "" + "name, "
            if not self.name
            else (
                "" + "role, "
                if not self.role
                else (
                    "" + "bio, "
                    if not self.bio
                    else (
                        "" + "links, "
                        if not self.links
                        else (
                            "" + "contacts, "
                            if not self.contacts
                            else (
                                "" + "experiences, "
                                if not self.experiences
                                else (
                                    "" + "projects, "
                                    if not self.projects
                                    else (
                                        "" + "educations, "
                                        if not self.educations
                                        else (
                                            "" + "skills, "
                                            if not self.skills
                                            else (
                                                "" + "certifications, "
                                                if not self.certifications
                                                else ""
                                            )
                                        )
                                    )
                                )
                            )
                        )
                    )
                )
            )
        )
        if err_str != "":
            raise ValueError("Education (" + err_str[:-2] + ") cannot be left empty!.")



from typing import List, Literal
from rich.console import Console, Group
from rich.table import Table
from rich.panel import Panel
from rich.text import Text
from rich.live import Live
from rich.spinner import Spinner
from rich.align import Align
from rich.markdown import Markdown
from rich.columns import Columns
from rich.rule import Rule
from rich import box
import time
from blank_portfolio.models.data_models import (
    Portfolio,
    Contact,
    Link,
    Skill,
    SkillLevel,
    Experience,
    Project,
)
from datetime import date
from blank_portfolio.services.cli_service import CLIService

console = Console()
cli_service = CLIService()


SKILL_LEVEL_COLORS = {
    SkillLevel.BEGINNER.name: "bright_black",
    SkillLevel.INTERMEDIATE.name: "yellow",
    SkillLevel.ADVANCED.name: "cyan",
    SkillLevel.EXPERT.name: "green",
}


def get_divider(text: str, style: str = "bright_black"):
    divider = Rule(text, style=style)
    return divider


def show_loading(sleep_time: float = 1.0):
    spinner = Spinner("dots", text="Loading portfolio data...", style="bold cyan")
    with Live(spinner, refresh_per_second=10, transient=True):
        time.sleep(sleep_time)


def format_portfolio_summary(portfolio: Portfolio) -> Panel:
    bio_md = "\n\n".join(portfolio.bio)
    bio_panel = Panel(Markdown(bio_md), title="Bio", border_style="magenta")
    header = Text(
        f"{portfolio.name}\n{portfolio.role}", justify="center", style="bold cyan"
    )
    return Panel(
        Group(header, bio_panel),
        title="Portfolio Summary",
        border_style="cyan",
        padding=(1, 2),
    )


def format_contacts(contacts: List[Contact]) -> Table:
    table = Table(title="Contacts", show_lines=True)
    table.add_column("Type", style="bold")
    table.add_column("Detail")
    for c in contacts:
        table.add_row(c.type.name, c.detail)
    return table


def format_links(links: List[Link]) -> Table:
    table = Table(title="Links", show_lines=True)
    table.add_column("Label", style="bold")
    table.add_column("URL", style="cyan underline")
    for link in links:
        table.add_row(link.label, link.url)
    return table


def format_skills(skills: List[Skill]) -> Table:
    table = Table(title="Skills")
    table.add_column("Skill", style="bold")
    table.add_column("Type")
    table.add_column("Level")
    for skill in skills:
        color = SKILL_LEVEL_COLORS.get(skill.level.name, "white")
        level_text = Text(skill.level.name.capitalize(), style=color)
        table.add_row(skill.name, skill.type.name.capitalize(), level_text)
    return table


def format_experiences(experiences: List[Experience]) -> Panel:
    aligned_panels = []
    for exp in experiences:
        start = exp.start_date.strftime("%b %Y") if exp.start_date else ""
        end = exp.end_date.strftime("%b %Y") if exp.end_date else "Present"
        header_text = f"{exp.role} @ {exp.company} ({start} - {end})"
        header = Text(header_text, style="bold cyan")

        # Format responsibilities with indentation and bullets
        body_lines = [f"- {r}" for r in exp.responsibilities]

        if exp.learnings:
            body_lines.append("")
            body_lines.append("[bold underline]Learnings:[/bold underline]")
            body_lines.extend(f"- {l}" for l in exp.learnings)

        body = "\n".join(body_lines)
        panel = Panel(
            body,
            title=header,
            border_style="blue",
            padding=(1, 2),  # Add padding for better spacing
            expand=False,
        )
        aligned_panels.append(Align(panel, align="center"))

    return Panel(
        Group(*aligned_panels),
        title="[bold bright_blue]Experiences[/bold bright_blue]",
        border_style="bright_blue",
        padding=(1, 2),
    )


def format_projects(projects: List[Project]) -> List[Panel]:
    panels = []
    for proj in projects:
        header = Text(proj.name, style="bold")
        techs = ", ".join(proj.technologies)
        body = f"{proj.description}\n\nTechnologies: {techs}"
        if proj.link:
            body += f"\nLink: [underline cyan]{proj.link}[/underline cyan]"
        panels.append(
            Align(
                Panel(body, title=header, border_style="green", expand=False),
                align="center",
            )
        )
    return Panel(
        Group(*panels), title="Projects", border_style="bright_green", padding=(1, 2)
    )


def print_splash():
    welcome_text = Text(
        "Welcome to Shantanu's CLI Portfolio", style="bold yellow", justify="center"
    )
    context = Text(
        "\nNavigate the portfolio by typing commands.",
        style="white italic",
        justify="center",
    )

    console.print(
        Panel(
            Align(
                Group(
                    welcome_text,
                    get_divider("----- --- -- - -- --- -----"),
                    context,
                ),
                align="center",
            ),
            title="[bold bright_green]🚀 Portfolio CLI[/bold bright_green]",
            border_style="bold magenta",
            padding=(1, 4),
        )
    )


def print_portfolio():
    show_loading(1.1)
    portfolio = cli_service.get_portfolio()
    console.print(format_portfolio_summary(portfolio))
    show_loading(0.4)
    contact_panel = format_contacts(portfolio.contacts)
    link_panel = format_links(portfolio.links)
    social_panel = Panel(
        Group(contact_panel, link_panel),
        title="Contacts & Links",
        border_style="magenta",
        padding=(1, 2),
    )
    skill_panel = format_skills(portfolio.skills)
    columns = Columns([skill_panel, social_panel], equal=True, expand=True)
    console.print(columns)
    show_loading(0.6)
    console.print(format_experiences(portfolio.experiences))
    show_loading(0.6)
    console.print(format_projects(portfolio.projects))


def print_skills(
    skill_name: List[str] = None,
    skill_type: List[str] = None,
    skill_level: List[str] = None,
    sort: Literal["asc", "desc", None] = None,
):
    skills = cli_service.get_skills_filtered(
        skill_names=skill_name,
        skill_types=skill_type,
        skill_levels=skill_level,
        sort=sort,
    )
    # skills = cli_service.get_skills_sorted_by_level(skills)
    show_loading(0.5)
    console.print("\n\n")
    skill_table = format_skills(skills)
    console.print(skill_table)
    console.print("\n\n")


def print_experiences():
    experiences = cli_service.get_experiences()
    exp_panels = []

    for exp in experiences:
        role_panel = Text(f"{exp.role}\n", style="bold cyan")
        date_panel = Text(
            f"{exp.start_date} to {exp.end_date if exp.end_date else 'Present'}\n",
            style="bold cyan",
        )
        company_panel = Text(f"At : {exp.company}", style="bold cyan")
        mode_panel = Text(f"Mode : {exp.mode.value}", style="bold cyan")

        col1 = Align(
            Panel(Group(role_panel, company_panel), box=box.MINIMAL), align="left"
        )
        col2 = Align(
            Panel(Group(date_panel, mode_panel), box=box.MINIMAL), align="right"
        )

        col_group = Columns(
            [col1, col2],
            align="",
            equal=False,
            expand=True,
        )

        responsibilites_panel = Panel(
            Group(
                get_divider("Responsibilites"),
                Text("\n"),
                Text(
                    "\n\n".join(f"- {r}" for r in exp.responsibilities), justify="left"
                ),
            ),
            style="cyan",
            box=box.MINIMAL,
        )

        learnings_panel = Panel(
            Group(
                get_divider("Learnings"),
                Text("\n"),
                Text("\n\n".join(f"- {r}" for r in exp.learnings), justify="left"),
            ),
            style="cyan",
            box=box.MINIMAL,
        )

        exp_panel = Panel(
            Group(
                col_group,
                responsibilites_panel,
                learnings_panel,
            ),
            title="_-'-_-'-_-'-_-'-_-'-_-'-_",
            style="bright_black",
        )
        exp_panels.append(exp_panel)

    parent_panel = Panel(
        Group(*exp_panels),
        title="Experiences",
        border_style="bright_blue",
        padding=(1, 2),
    )
    show_loading(0.4)
    console.print(parent_panel)
